@@ -2,6 +2,7 @@
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
 
 import atexit
 import errno
@@ -10,10 +11,11 @@ import os
 import signal
 import sys
 import traceback
+import typing as t
 from functools import partial
 from io import FileIO, TextIOWrapper
 from logging import StreamHandler
-from typing import Optional
+from pathlib import Path
 
 import zmq
 from IPython.core.application import (  # type:ignore[attr-defined]
@@ -132,7 +134,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     poller = Any()  # don't restrict this even though current pollers are all Threads
     heartbeat = Instance(Heartbeat, allow_none=True)
 
-    context: Optional[zmq.Context] = Any()  # type:ignore[assignment]
+    context: zmq.Context[t.Any] | None = Any()  # type:ignore[assignment]
     shell_socket = Any()
     control_socket = Any()
     debugpy_socket = Any()
@@ -160,10 +162,9 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
 
     @property
     def abs_connection_file(self):
-        if os.path.basename(self.connection_file) == self.connection_file:
-            return os.path.join(self.connection_dir, self.connection_file)
-        else:
-            return self.connection_file
+        if Path(self.connection_file).name == self.connection_file and self.connection_dir:
+            return str(Path(str(self.connection_dir)) / self.connection_file)
+        return self.connection_file
 
     # streams, etc.
     no_stdout = Bool(False, help="redirect stdout to the null device").tag(config=True)
@@ -230,7 +231,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             if port <= 0:
                 port = 1
                 path = "%s-%i" % (self.ip, port)
-                while os.path.exists(path):
+                while Path(path).exists():
                     port = port + 1
                     path = "%s-%i" % (self.ip, port)
             else:
@@ -256,6 +257,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
                     raise
                 if attempt == max_attempts - 1:
                     raise
+        return None
 
     def write_connection_file(self):
         """write connection info to JSON file"""
@@ -270,7 +272,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             iopub_port=self.iopub_port,
             control_port=self.control_port,
         )
-        if os.path.exists(cf):
+        if Path(cf).exists():
             # If the file exists, merge our info into it. For example, if the
             # original file had port number 0, we update with the actual port
             # used.
@@ -290,7 +292,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         cf = self.abs_connection_file
         self.log.debug("Cleaning up connection file: %s", cf)
         try:
-            os.remove(cf)
+            Path(cf).unlink()
         except OSError:
             pass
 
@@ -305,14 +307,14 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         except OSError:
             self.log.debug("Connection file not found: %s", self.connection_file)
             # This means I own it, and I'll create it in this directory:
-            os.makedirs(os.path.dirname(self.abs_connection_file), mode=0o700, exist_ok=True)
+            Path(self.abs_connection_file).parent.mkdir(mode=0o700, exist_ok=True, parents=True)
             # Also, I will clean it up:
             atexit.register(self.cleanup_connection_file)
             return
         try:
             self.load_connection_file()
         except Exception:
-            self.log.error(
+            self.log.error(  # noqa: G201
                 "Failed to load connection file: %r", self.connection_file, exc_info=True
             )
             self.exit(1)
@@ -422,10 +424,10 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
 
     def log_connection_info(self):
         """display connection info, and store ports"""
-        basename = os.path.basename(self.connection_file)
+        basename = Path(self.connection_file).name
         if (
             basename == self.connection_file
-            or os.path.dirname(self.connection_file) == self.connection_dir
+            or str(Path(self.connection_file).parent) == self.connection_dir
         ):
             # use shortname
             tail = basename
@@ -443,7 +445,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             self.log.info(line)
         # also raw print to the terminal if no parent_handle (`ipython kernel`)
         # unless log-level is CRITICAL (--quiet)
-        if not self.parent_handle and int(self.log_level) < logging.CRITICAL:
+        if not self.parent_handle and int(self.log_level) < logging.CRITICAL:  # type:ignore[call-overload]
             print(_ctrl_c_message, file=sys.__stdout__)
             for line in lines:
                 print(line, file=sys.__stdout__)
@@ -459,7 +461,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     def init_blackhole(self):
         """redirects stdout/stderr to devnull if necessary"""
         if self.no_stdout or self.no_stderr:
-            blackhole = open(os.devnull, "w")  # noqa
+            blackhole = open(os.devnull, "w")  # noqa: SIM115
             if self.no_stdout:
                 sys.stdout = sys.__stdout__ = blackhole  # type:ignore[misc]
             if self.no_stderr:
@@ -622,50 +624,13 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         handler.setFormatter(formatter)
         logger.addHandler(handler)
 
-    def _init_asyncio_patch(self):
-        """set default asyncio policy to be compatible with tornado
-
-        Tornado 6 (at least) is not compatible with the default
-        asyncio implementation on Windows
-
-        Pick the older SelectorEventLoopPolicy on Windows
-        if the known-incompatible default policy is in use.
-
-        Support for Proactor via a background thread is available in tornado 6.1,
-        but it is still preferable to run the Selector in the main thread
-        instead of the background.
-
-        do this as early as possible to make it a low priority and overridable
-
-        ref: https://github.com/tornadoweb/tornado/issues/2608
-
-        FIXME: if/when tornado supports the defaults in asyncio without threads,
-               remove and bump tornado requirement for py38.
-               Most likely, this will mean a new Python version
-               where asyncio.ProactorEventLoop supports add_reader and friends.
-
-        """
-        if sys.platform.startswith("win") and sys.version_info >= (3, 8):
-            import asyncio
-
-            try:
-                from asyncio import WindowsProactorEventLoopPolicy, WindowsSelectorEventLoopPolicy
-            except ImportError:
-                pass
-                # not affected
-            else:
-                if type(asyncio.get_event_loop_policy()) is WindowsProactorEventLoopPolicy:
-                    # WindowsProactorEventLoopPolicy is not compatible with tornado 6
-                    # fallback to the pre-3.8 default of Selector
-                    asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())
-
     def init_pdb(self):
         """Replace pdb with IPython's version that is interruptible.
 
         With the non-interruptible version, stopping pdb() locks up the kernel in a
         non-recoverable state.
         """
-        import pdb  # noqa
+        import pdb
 
         from IPython.core import debugger
 
@@ -678,7 +643,6 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     @catch_config_error
     def initialize(self, argv=None):
         """Initialize the application."""
-        self._init_asyncio_patch()
         super().initialize(argv)
         if self.subapp is not None:
             return
@@ -700,8 +664,8 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         except Exception:
             # Catch exception when initializing signal fails, eg when running the
             # kernel on a separate thread
-            if int(self.log_level) < logging.CRITICAL:
-                self.log.error("Unable to initialize signal:", exc_info=True)
+            if int(self.log_level) < logging.CRITICAL:  # type:ignore[call-overload]
+                self.log.error("Unable to initialize signal:", exc_info=True)  # noqa: G201
         self.init_kernel()
         # shell init steps
         self.init_path()
